﻿import QtQuick 2.7
import QtQuick.Controls 1.5
import QtQuick.Layouts 1.0
import QtQml.Models 2.2
import QtQuick.Controls.Styles 1.4

import "../../helpers" as Helpers

Item {
	id: root

	signal itemSelected()
	signal componentCompleted()

	Layout.fillHeight: true
	Layout.fillWidth:  true

	property int itemHeight: 32
	property int expanderWidth: 32
	property int itemFontSize: 20

	property alias busyIndicator: busyIndicator
	property alias firstColumn: firstColumn
	property alias secondColumn: secondColumn
	property alias itemsTree: itemsTree
	property alias popup: popup

	property alias currentIndex: itemsTree.currentIndex

	function reset() {
		itemsTree.reset()
	}
	function filter(query) {
		itemsTree.filter(query)
	}

	Helpers.MyPopup {
		id: popup
	}

	property var itemModel: ListModel{}

	// A lot of race-condition bug-sources:

	Timer {
		id: singleClickedStopTimer
		interval: 100

		onTriggered: {
			if (singleClicked.running) {
				singleClicked.stop()
			}
		}
	}

	Timer {
		id: singleClicked
		interval: 200

		onTriggered: {
			itemSelected()
			itemModel.selectionChanged()
		}
	}

	Timer {
		id: itemSelectedTimer
		interval: 200

		onTriggered: {
			itemSelected()
			itemModel.selectionChanged()
		}
	}

	Timer {
		id: retryReload
		interval: 1000

		onTriggered: itemModel.reload()
	}

	// The tree:

	TreeView {
		id: itemsTree

		model: itemModel
		sortIndicatorVisible: true
		//rootIndex: rootPathIndex
		//model: fileSystemModel
		//rootIndex: rootPathIndex

		property alias busyIndicator: busyIndicator
		property alias firstColumn: firstColumn
		property alias secondColumn: secondColumn
		property string filterQuery: ""

		anchors.fill: parent

		function expandRecursively(index) {
			var parent = itemModel.getParentIndex(index)
			if (itemModel.isValidIndex(parent)) {
				expandRecursively(parent)
			}
			expand(index)
		}
		function collapseExpandAll(shouldCollapse) {
			var everyIndex = itemModel.getIndexesByFieldValue_recursive("Name", "", 1, itemModel.getRootIndex())
			var everyIndexLength = everyIndex.length
			for (var i = 0; i < everyIndexLength; i++) {
				var index = everyIndex[i]
				if (shouldCollapse) {
					collapse(index)
				} else {
					expand(index)
				}
			}
		}
		function collapseAll() {
			collapseExpandAll(true)
		}
		function expandAll() {
			collapseExpandAll(false)
		}

		function reset() {
			collapseAll();
			showAll();
			var mephiIndex = itemModel.index(0,0)
			//collapse(mephiIndex)
			expand(mephiIndex)
			/*var mephiChildrenIndexes = itemModel.getChildrenIndexes(mephiIndex)
			for (var i=0; i < mephiChildrenIndexes.length; i++) {
				collapse(mephiChildrenIndexes[i]);
			}*/

			var affiliatesIndex = itemModel.index(1,0)
			//collapse(affiliatesIndex)
			expand(affiliatesIndex)
			/*var affiliatesChildrenIndexes = itemModel.getChildrenIndexes(affiliatesIndex)
			for (var j=0; j < affiliatesChildrenIndexes.length; j++) {
				collapse(affiliatesChildrenIndexes[j]);
			}*/

			flickableItem.contentY = -100
		}
		function filter(query) {
			if (filterTimer.running) {
				filterTimer.stop()
			}
			if (query.length < 2) {
				reset()
				return
			}

			filterQuery = query
			filterTimer.start()
		}

		function hideAll() {
			itemModel.hideAll();
		}
		function showAll() {
			itemModel.showAll();
		}

		function show(index) {
			itemModel.showIndex(index);
		}
		function hide(index) {
			itemModel.hideIndex(index);
		}

		Timer {
			id: filterTimer
			interval: 300

			onTriggered: {
				itemsTree.collapseAll()
				itemsTree.hideAll()
				var toExpand = itemModel.findByQuery(itemsTree.filterQuery)
				var toExpandLength = toExpand.length
				for (var i = 0; i < toExpandLength; i++) {
					var index = toExpand[i]
					itemsTree.expandRecursively(index)
					itemsTree.show(index)
				}
			}
		}

		BusyIndicator {
			id: busyIndicator
			x: -1
			y: 150
			anchors.fill: parent
		}

		rowDelegate: Rectangle {
			property string idFieldName: itemModel.idField
			property string itemId: typeof(model) == "undefined" ? "none" : ( model == null ? "none" : model[idFieldName] )
			visible: itemId == "none" ? false : !itemModel.isHiddenIndex(itemModel.findIndexById(itemId))
			enabled: visible
			height: visible ? itemHeight : 0
			color: styleData.selected ? "skyblue" : styleData.alternate ? "white" : "#f4f8fa"
		}
		itemDelegate: Item {
			Text {
				visible: !itemModel.isHiddenIndex(styleData.index)
				height: visible ? itemHeight: 0
				font.pixelSize: itemFontSize
				anchors.fill: parent
				color: "black"
				elide: styleData.elideMode
				text: visible ? styleData.value : ""
				verticalAlignment: Text.AlignVCenter
			}
		}
		style: TreeViewStyle {
			branchDelegate: Rectangle {
				visible: !itemModel.isHiddenIndex(styleData.index)
				width: visible ? expanderWidth: 0
				height: visible ? itemHeight: 0
				color: "transparent"
				Text {
					width: expanderWidth
					height: itemHeight
					font.pixelSize: itemFontSize
					font.bold: true
					text: styleData.depth<=0?"":styleData.isExpanded ? "-" : "+"
					verticalAlignment: Text.AlignVCenter
					horizontalAlignment: Text.AlignHCenter
				}
			}
		}

		TableViewColumn {
			id: firstColumn
			width: parent.width - 17 - (secondColumn.visible ? secondColumn.width : 0)
		}

		TableViewColumn {
			id: secondColumn
			visible: false
			elideMode: Text.ElideRight
		}

		Component.onCompleted: {
			//itemModel.reload()
			componentCompleted()
			unitsTree.codeStrColumn.resizeToContents()
		}

		onExpanded: {
			singleClickedStopTimer.start()
		}
		onCollapsed: {
			singleClickedStopTimer.start()
		}
		onCurrentIndexChanged: {
			if (singleClicked.running) {
				singleClicked.stop()
			}
			expand(currentIndex)
			if (itemModel.isValidIndex(currentIndex) && itemModel.isClickableIndex(currentIndex)) {
				itemSelectedTimer.start()
			}
		}
		selection: ItemSelectionModel {
			model: itemModel
		}
		onClicked: {
			//expand(index);
			if (index !== currentIndex) {
				selection.setCurrentIndex(index, ItemSelectionModel.ClearAndSelect);
				if (itemModel.isClickableIndex(index)) {
					singleClicked.start()
				}
			}
		}
		onPressAndHold: {
		}
		onDoubleClicked: {
			if (itemSelectedTimer.running) {
				itemSelectedTimer.stop()
			}
			//console.log("is expanded: "+ isExpanded(index))
			if (isExpanded(index)) {
				collapse(index)
			} else {
				expand(index)
			}
			if (singleClicked.running) {
				singleClicked.stop()
			}
		}
	}
}
